package cd.casic.module.machine.service.impl;

import cd.casic.framework.commons.pojo.PageResult;
import cd.casic.framework.commons.util.object.BeanUtils;
import cd.casic.module.machine.controller.vo.SecretKeyVO;
import cd.casic.module.machine.dal.dataobject.MachineInfoDO;
import cd.casic.module.machine.dal.dataobject.SecretKeyDO;
import cd.casic.module.machine.dal.mysql.SecretKeyMapper;
import cd.casic.module.machine.service.MachineInfoService;
import cn.hutool.core.io.resource.ResourceUtil;
import cn.hutool.core.util.IdUtil;
import cd.casic.module.machine.utils.AliYunOssClient;
import cd.casic.module.machine.service.SecretKeyService;
import com.google.common.annotations.VisibleForTesting;
import jakarta.annotation.Resource;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

import static cd.casic.framework.commons.exception.util.ServiceExceptionUtil.exception;
import static cd.casic.module.machine.contants.MachineErrorCodeConstants.*;

/**
 * 密钥服务实现类
 */
@Service("secretKeyService")
public class SecretKeyServiceImpl implements SecretKeyService {

    @Resource
    private MachineInfoService machineInfoService;

    @Resource
    private SecretKeyMapper secretKeyMapper;

    @Override
    public void deleteSecretKey(Long id) {
        validateSecretKeyExists(id);
        secretKeyMapper.deleteById(id);
    }

    @Override
    public SecretKeyVO getSecretKey(Long id) {
        SecretKeyDO secretKeyDO = validateSecretKeyExists(id);
        return BeanUtils.toBean(secretKeyDO, SecretKeyVO.class);
    }

    @Override
    public void unbindMachine(SecretKeyVO secretKeyVO) {
        machineInfoService.bindingSecretKey(secretKeyVO.getMachineInfoIds(),null);
    }

    @Override
    public List<MachineInfoDO> getBindMachine(Long secretKeyId) {
        return machineInfoService.selectBindMachineBySecretKey(secretKeyId);
    }

    @Override
    public Long createSecretKey(SecretKeyVO secretKeyVO) {
        validateSecretKeyAdd(secretKeyVO);
        SecretKeyDO secretKeyDO = BeanUtils.toBean(secretKeyVO, SecretKeyDO.class);
        secretKeyMapper.insert(secretKeyDO);
        return secretKeyDO.getId();


    }

    @Override
    public void updateSecretKey(SecretKeyVO secretKeyVO) {
        SecretKeyDO secretKeyDO = validateSecretKeyExists(secretKeyVO.getId());
        BeanUtils.copyProperties(secretKeyVO, secretKeyDO);
        secretKeyMapper.updateById(secretKeyDO);
    }

    @Override
    public void bindingMachine(SecretKeyVO secretKeyVO) {
        validateSecretKeyExists(secretKeyVO.getId());
        machineInfoService.bindingSecretKey(secretKeyVO.getMachineInfoIds(), secretKeyVO.getId());
    }

    @Override
    @Transactional
    public void deleteSecretKeyList(List<Long> ids) {
        machineInfoService.bindingSecretKey(ids,null);
        secretKeyMapper.deleteBatchIds(ids);
    }

    @Override
    public PageResult<SecretKeyDO> getSecretKeypage(SecretKeyVO secretKeyVO) {
        return secretKeyMapper.selectPage(secretKeyVO);
    }


    @VisibleForTesting
    void validateSecretKeyAdd(SecretKeyVO secretKeyVO) {
        if (secretKeyVO == null) {
            throw exception(SECRET_KEY_NULL);
        }
    }



    @VisibleForTesting
    SecretKeyDO validateSecretKeyExists(Long id) {
        if (id == null) {
            throw exception(SECRET_KEY_NOT_EXISTS);
        }
        SecretKeyDO secretKeyDO = secretKeyMapper.selectById(id);
        if (secretKeyDO == null) {
            throw exception(SECRET_KEY_NOT_EXISTS);
        }
        return secretKeyDO;
    }

}
